package cn.flutter.pluginuuid

import android.content.Context
import android.content.Context.TELEPHONY_SERVICE
import android.os.Build
import android.provider.Settings
import android.telephony.TelephonyManager
import android.text.TextUtils
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import java.net.NetworkInterface
import java.net.SocketException
import java.util.*

/** PluginuuidPlugin */
public class PluginuuidPlugin : FlutterPlugin, MethodCallHandler {

    lateinit var context: Context

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        val channel = MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "pluginuuid")
        print("uuid_onattach")
        channel.setMethodCallHandler(PluginuuidPlugin());
        context = flutterPluginBinding.applicationContext
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "pluginuuid")
            print("registrar_onattach")
            channel.setMethodCallHandler(PluginuuidPlugin())
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {

        if (call.method == "uuid") {
            result.success(getUUid())
        } else {
            result.notImplemented()
        }
    }

    fun getUUid():String? {
      var deviceIdQ: String?=null
        if (Build.VERSION.SDK_INT > Build.VERSION_CODES.P) {
            try { //获取设备的MACAddress地址 去掉中间相隔的冒号
                deviceIdQ = Settings.Secure.getString(context.contentResolver, Settings.Secure.ANDROID_ID)

                if (TextUtils.isEmpty(deviceIdQ)) {
                    deviceIdQ = getLocalMac(context)?.replace(":", "")
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
            //如果以上搜没有获取相应的则自己生成相应的UUID作为相应设备唯一标识符
            if (TextUtils.isEmpty(deviceIdQ)) {
                val uuid = UUID.randomUUID()
                deviceIdQ = uuid.toString().replace("-", "")
            }
        }else{
          //权限申请
          val telephonyManager = context
                  .getSystemService(TELEPHONY_SERVICE) as TelephonyManager
          deviceIdQ = telephonyManager.deviceId
        }
      return deviceIdQ
    }

    fun getLocalMac(context: Context): String? {
        val macAddress: String
        val buf = StringBuilder()
        var networkInterface: NetworkInterface?
        try {
            networkInterface = NetworkInterface.getByName("eth1")
            if (networkInterface == null) {
                networkInterface = NetworkInterface.getByName("wlan0")
            }
            if (networkInterface == null) {
                return ""
            }
            val addr = networkInterface.hardwareAddress
            for (b in addr) {
                buf.append(String.format("%02X:", b))
            }
            if (buf.length > 0) {
                buf.deleteCharAt(buf.length - 1)
            }
            macAddress = buf.toString()
        } catch (e: SocketException) {
            e.printStackTrace()
            return ""
        }
        return macAddress
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    }
}
